using System;
using System.Data;
using System.Data.SqlClient;

class DataSetTableRowColumn {
    SqlConnection dbConnection;
	SqlCommand selCommandCust, selCommandOrd;
	SqlDataAdapter dbAdapterOrd, dbAdapterCust;
	DataSet ds = new DataSet();
	DataTable dtCustomers, dtOrders;
	DataRelation coRel;

	DataSetTableRowColumn() {
		dbConnection = new SqlConnection(@"Data Source=(local)\SQLEXPRESS;" +
			@"AttachDbFilename=E:\Data\NORTHWND.MDF;" +
			"Integrated Security=True;User Instance=True");

		selCommandCust = new SqlCommand(
			"SELECT CustomerID, CompanyName, Country FROM Customers", dbConnection);
		selCommandOrd = new SqlCommand(
			"SELECT OrderID, CustomerID, ShipCountry FROM Orders", dbConnection);

		dbAdapterCust = new SqlDataAdapter(selCommandCust);
		dbAdapterOrd = new SqlDataAdapter(selCommandOrd);

		// Die Adapter werden angewiesen, Schema-Informationen zu bernehmen:
		dbAdapterCust.MissingSchemaAction = MissingSchemaAction.AddWithKey;
		dbAdapterOrd.MissingSchemaAction = MissingSchemaAction.AddWithKey;

		try {
			dbConnection.Open(); // Vermeidet zweimaliges ffnen und Schlieen durch Fill() und FillSchema()
			dbAdapterCust.Fill(ds, "Customers");
			dbAdapterOrd.Fill(ds, "Orders");
		} finally {
			dbConnection.Close();
		}
		
		dtCustomers = ds.Tables["Customers"];
		dtOrders = ds.Tables["Orders"];

		// DataTable-Ereignisse
		dtCustomers.ColumnChanging += CustomersOnColumnChanging;
		dtCustomers.ColumnChanged += CustomersOnColumnChanged;
		dtCustomers.RowChanging += CustomersOnRowChanging;
		dtCustomers.RowChanged += CustomersOnRowChanged;

		// Vereinbarung einer Master-Detail - Beziehung:
		DataColumn parent = dtCustomers.Columns["CustomerID"];
		DataColumn child = dtOrders.Columns["CustomerID"];
		ds.Relations.Add(new DataRelation("CustomersOrders", parent, child));
		coRel = ds.Relations["CustomersOrders"];

		// Der SqlCommandBuilder erstellt automatisch die vom DataAdapter bentigten SQL-Kommandos
		// zum Zum Aktualisieren der Datenbank. Wir ndern nur die Customers-Tabelle.
		new SqlCommandBuilder(dbAdapterCust);
	}

	void PrintDataCustomers() {
		Console.WriteLine("Die ersten 10 Flle aus der Customers-Tabelle:\n");
		Console.WriteLine("{0,15} {1,40} {2,20}", "CustomerID", "CompanyName", "Country");
		Console.WriteLine("{0,15} {1,40} {2,20}\n", dtCustomers.Rows[0]["CustomerID"].GetType(),
			dtCustomers.Rows[0]["CompanyName"].GetType(),
			dtCustomers.Rows[0]["Country"].GetType());
		for (int i = 0; i < 10; i++)
			if (dtCustomers.Rows[i].RowState != DataRowState.Deleted)
				Console.WriteLine("{0,15} {1,40} {2,20}", dtCustomers.Rows[i][0],
					dtCustomers.Rows[i][1], dtCustomers.Rows[i]["Country"]);

		// Schema-Informationen zur Customers-Tabelle anzeigen
		Console.WriteLine("\nSchema-Informationen zur Customers-Tabelle:");
		// Spaltenattribute
		Console.WriteLine(" MaxLength(CompanyName)     = " + dtCustomers.Columns["CompanyName"].MaxLength +
			"\n Primrschlssel(Customers) = " + dtCustomers.PrimaryKey[0].ColumnName + "\n");
		// Constaints
		foreach (Constraint c in dtCustomers.Constraints) {
			Console.WriteLine(" " + c.ConstraintName + "   " + c.GetType());
			if (c.GetType() == typeof(UniqueConstraint))
				Console.WriteLine("  Spalte zur UniqueConstraint: "+(c as UniqueConstraint).Columns[0].ColumnName);
		}
	}

	void PrintDataOrders() {
		Console.WriteLine("\n\nDie ersten 5 Flle aus der Orders-Tabelle:\n");
		Console.WriteLine("{0,15} {1,15} {2,20}", "OrderID", "CustomerID", "ShipCountry");
		Console.WriteLine("{0,15} {1,15} {2,20}\n", dtOrders.Rows[0]["OrderID"].GetType(),
			dtOrders.Rows[0]["CustomerID"].GetType(),
			dtOrders.Rows[0]["ShipCountry"].GetType());
		for (int i = 0; i < 5; i++)
			Console.WriteLine("{0,15} {1,15} {2,20}", dtOrders.Rows[i][0],
				dtOrders.Rows[i][1], dtOrders.Rows[i]["ShipCountry"]);

		// Constaints zur Orders-Tabelle
		Console.WriteLine("\nConstaints zur Orders-Tabelle:");
		foreach (Constraint c in dtOrders.Constraints) {
			Console.WriteLine(" " + c.ConstraintName + "   " + c.GetType());
			if (c.GetType() == typeof(UniqueConstraint))
				Console.WriteLine("  Spalte zur UniqueConstraint:          " + (c as UniqueConstraint).Columns[0].ColumnName);
			if (c.GetType() == typeof(ForeignKeyConstraint))
				Console.WriteLine("  Spalte zu einer ForeignKeyConstraint: " + (c as ForeignKeyConstraint).Columns[0].ColumnName);
		}
		//Console.WriteLine("\nds.Relations.Count: " + ds.Relations.Count);
	}

	void PrintMasterDetailConstraints() {
		Console.WriteLine("\n\nAus der Master-Details-Beziehung resultierende Constaints:");
		Console.WriteLine("\nParentKeyConstraint zur Beziehung \"CustomersOrders\":" +
			"\n Tabelle: " + coRel.ParentKeyConstraint.Table.TableName +
			"\n Feld:    " + coRel.ParentKeyConstraint.Columns[0].ColumnName +
			"\n Typ:     " + coRel.ParentKeyConstraint.GetType());
		Console.WriteLine("\nChildKeyConstraint zur Beziehung \"CustomersOrders\":" +
			"\n Tabelle: " + coRel.ChildKeyConstraint.Table.TableName +
			"\n Feld:    " + coRel.ChildKeyConstraint.Columns[0].ColumnName +
			"\n Typ:     " + coRel.ChildKeyConstraint.GetType());
	}

	void PrintCustomerChildRows() {
		DataRow[] crs;
		Console.WriteLine("\n\nDie ersten beiden Flle aus der Customers-Tabelle mit zugeh. Bestellungen:");
		for (int i = 0; i < 2; i++)
			if (dtCustomers.Rows[i].RowState != DataRowState.Deleted) {
				Console.WriteLine("\nCustomerID: " + dtCustomers.Rows[i][0] + "  CompanyName: " + dtCustomers.Rows[i][1]);
				crs = dtCustomers.Rows[i].GetChildRows(ds.Relations["CustomersOrders"]);
				foreach(DataRow dr in crs)
					Console.WriteLine("   OrderID: "+dr["OrderID"]+"  ShipCountry: "+ dr["ShipCountry"]);
			}
	}

	void ChangeData() {
		//DataRow dr = dtCustomers.Rows[0];
		DataRow dr = dtCustomers.Rows.Find("ALFKI");

		// Einzelne Werte ndern
		//Console.WriteLine("\nVor BeginEdit: " + dr["Country"]);
		dr.BeginEdit();
		dr["CompanyName"] = "Alfreds Kutterkiste";
		dr["Country"] = "Hummerland";
		//dr["Country"] = DBNull.Value;
		//Console.WriteLine("Vor End- bzw. CancelEdit: " + dr["Country"]);
		if (dr.HasErrors == false)
		    dr.EndEdit();
		else
		    dr.CancelEdit();
		//Console.WriteLine("Nach End- bzw. CancelEdit: " + dr["Country"] + "\n");

		// Zeile lschen oder ergnzen
		DataRow drem = dtCustomers.Rows.Find("BONAS");
		if (drem != null) {
			drem.Delete();
			//dtCustomers.Rows.Remove(drem); // fhrt nicht zum Lschen der zugeh. Datenbankzeile beim Update() 
		} else {
			drem = dtCustomers.NewRow(); // neue Zeile mit dem dtCustomers-Schema erstellen
			drem["CustomerID"] = "BONAS";
			drem["CompanyName"] = "Bonner Assekuranz";
			drem["Country"] = "Germany";
			dtCustomers.Rows.Add(drem);  // Zeile in die Tabelle dtCustomers aufnehmen
		}
	}

	void CustomersOnColumnChanging(Object sender, DataColumnChangeEventArgs e) {
		DataTable table = (DataTable)sender;
		Console.WriteLine("\nColumnChanging,\tSpalte: " + e.Column.Caption +
			"\n  vorgeschl. Wert: " + e.ProposedValue + ", akt. Wert: " + e.Row[e.Column]);
		if (e.Column.ColumnName.Equals("Country"))
			if ((e.ProposedValue as String).Equals("Kummerland"))
				e.Row.SetColumnError(e.Column, "Error");
				//	e.ProposedValue = e.Row[e.Column];
	}

	void CustomersOnColumnChanged(Object sender, DataColumnChangeEventArgs e) {
		DataTable table = (DataTable)sender;
		Console.WriteLine("\nColumnChanged,\tSpalte: " + e.Column.Caption +
			"\n  vorgeschl. Wert: " + e.ProposedValue + ", akt. Wert: " + e.Row[e.Column]);
	}

	void CustomersOnRowChanging(Object sender,DataRowChangeEventArgs e) {
		DataTable table = (DataTable)sender;
		Console.WriteLine("\nRowChanging (ID: " + e.Row.ItemArray[0] +
			"), Action: " + e.Action + ", RowState: " + e.Row.RowState);
	}

	void CustomersOnRowChanged(Object sender, DataRowChangeEventArgs e) {
		DataTable table = (DataTable)sender;
		Console.WriteLine("\nRowChanged (ID: " + e.Row.ItemArray[0] +
			"), Action: " + e.Action + ", RowState: " + e.Row.RowState);
	}

	
	void UpdateData() {
		dbAdapterCust.Update(dtCustomers);
	}

	static void Main() {
		DataSetTableRowColumn ssd = new DataSetTableRowColumn();
		//ssd.PrintDataCustomers();
		//ssd.PrintDataOrders();
		//ssd.PrintMasterDetailConstraints();
		//ssd.PrintCustomerChildRows();
		ssd.ChangeData();
		//ssd.UpdateData();
		Console.ReadLine();
	}
}