﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Reflection;
using Microsoft.Win32;

namespace Eddie {
	/// <summary>
	/// Interaktionslogik für MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window {

		string strUserKey, strWordWrap, strVertSB, strZoomFactor;

		bool wordWrap = true;
		bool vertSB = false;
		int zoomFactor = 100;
		String searchText;
		TextRange tr;

		public MainWindow() {
			InitializeComponent();
			// Assembly-Attribute ermitteln
			System.Reflection.Assembly ass = System.Reflection.Assembly.GetExecutingAssembly();
			AssemblyCompanyAttribute compName = (AssemblyCompanyAttribute) Attribute.GetCustomAttribute(ass, typeof(AssemblyCompanyAttribute));
			AssemblyProductAttribute prodName = (AssemblyProductAttribute)Attribute.GetCustomAttribute(ass, typeof(AssemblyProductAttribute));
			strUserKey = "Software\\" +
					compName.Company + "\\" +
					prodName.Product + "\\" +
					ass.GetName().Version;
			strWordWrap = "WordWrap";
			strVertSB = "VertSB";
			strZoomFactor = "ZoomFactor";
		}

		void activateUserSettings() {
			if (wordWrap) {
				eddie.HorizontalScrollBarVisibility = ScrollBarVisibility.Disabled;
				eddie.Document.PageWidth = Double.NaN;
			} else {
				eddie.HorizontalScrollBarVisibility = ScrollBarVisibility.Visible;
				eddie.Document.PageWidth = 5000;
			}
			if (vertSB)
				eddie.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;
			else
				eddie.VerticalScrollBarVisibility = ScrollBarVisibility.Disabled;
			eddie.LayoutTransform = new ScaleTransform(zoomFactor / 100, zoomFactor / 100);
		}

		private void miOptions_Click(object sender, RoutedEventArgs e) {
			OptionsDialog od = new OptionsDialog();
			od.Owner = this;
			od.WordWrap = wordWrap;
			od.VertSB = vertSB;
			od.Zoom = zoomFactor;
			if (od.ShowDialog() == true) {
				wordWrap = od.WordWrap;
				vertSB = od.VertSB;
				zoomFactor = od.Zoom;
			}
			activateUserSettings();
		}

		private void miSearch_Click(object sender, RoutedEventArgs e) {
			SearchDialog sd = new SearchDialog();
			sd.FindNext += this.searchDialog_FindNext;
			sd.Closed += this.searchDialog_Closed;
			sd.Owner = this;
			sd.Show();
		}

		private void searchDialog_FindNext(object sender, EventArgs e) {
			searchText = ((SearchDialog)sender).SearchText;
			if (tr != null)
				tr.ApplyPropertyValue(TextElement.BackgroundProperty, new SolidColorBrush(Colors.White));
			FindAndReplaceManager frm = new FindAndReplaceManager(eddie.Document);
			TextPointer tp = eddie.CaretPosition;
			tr = frm.GetTextRangeFromPosition(ref tp, searchText, FindOptions.None);
			if (tr != null) {
				tr.ApplyPropertyValue(TextElement.BackgroundProperty, new SolidColorBrush(Colors.Yellow));
				eddie.CaretPosition = tp;
			} else
				eddie.CaretPosition = tp.DocumentStart;
		}

		private void searchDialog_Closed(object sender, EventArgs ea) {
			if (tr != null)
				tr.ApplyPropertyValue(TextElement.BackgroundProperty, new SolidColorBrush(Colors.White));
		}

		private void Window_Loaded(object sender, RoutedEventArgs e) {
			Keyboard.Focus(eddie);

			RegistryKey userKey = Registry.CurrentUser.OpenSubKey(strUserKey);
			if (userKey != null) {
				try {
					int ww = (int) userKey.GetValue(strWordWrap);
					switch (ww) {
						case 0: wordWrap = false; break;
						case 1: wordWrap = true; break;
					}
					int vsb = (int) userKey.GetValue(strVertSB);
					switch (vsb) {
						case 0: vertSB = false; break;
						case 1: vertSB = true; break;
					}
					int zf = (int) userKey.GetValue(strZoomFactor);
					if (zf >= 100 && zf <= 500)
						zoomFactor = zf;
				} catch {
					MessageBox.Show("Fehler beim Laden der Einstellungen", Title,
						MessageBoxButton.OK, MessageBoxImage.Error);
				}
				userKey.Close();
				activateUserSettings();
			}
		}

		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			RegistryKey userKey = null;
			try {
				userKey = Registry.CurrentUser.OpenSubKey(strUserKey, true);
				if (userKey == null)
					userKey = Registry.CurrentUser.CreateSubKey(strUserKey);
				userKey.SetValue(strWordWrap, (wordWrap == true ? 1 : 0));
				userKey.SetValue(strVertSB, (vertSB == true ? 1 : 0));
				userKey.SetValue(strZoomFactor, zoomFactor);
			} catch {
				MessageBox.Show("Fehler beim Speichern der Einstellungen", Title,
					MessageBoxButton.OK, MessageBoxImage.Error);
			} finally {
				if (userKey != null)
					userKey.Close();
			}
		}
	}
}
