﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Configuration;

namespace Eddie {
	/// <summary>
	/// Interaktionslogik für MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window {

		Settings applicationSettings;

		bool wordWrap = true;
		bool vertSB = false;
		int zoomFactor = 100;

		String searchText;
		TextRange tr;

		public MainWindow() {
			InitializeComponent();
			applicationSettings = new Settings();
			applicationSettings.SettingChanging += new SettingChangingEventHandler(applicationSettings_SettingChanging);
		}

		void activateUserSettings() {
			if (wordWrap) {
				eddie.HorizontalScrollBarVisibility = ScrollBarVisibility.Disabled;
				eddie.Document.PageWidth = Double.NaN;
			} else {
				eddie.HorizontalScrollBarVisibility = ScrollBarVisibility.Visible;
				eddie.Document.PageWidth = 5000;
			}
			if (vertSB)
				eddie.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;
			else
				eddie.VerticalScrollBarVisibility = ScrollBarVisibility.Disabled;
			eddie.LayoutTransform = new ScaleTransform(zoomFactor / 100, zoomFactor / 100);
		}


		void miOptions_Click(object sender, RoutedEventArgs e) {
			OptionsDialog od = new OptionsDialog();
			od.Owner = this;
			od.WordWrap = wordWrap;
			od.VertSB = vertSB;
			od.Zoom = zoomFactor;
			if (od.ShowDialog() == true) {
				wordWrap = od.WordWrap;
				vertSB = od.VertSB;
				zoomFactor = od.Zoom;
			}
			activateUserSettings();
		}

		void miSearch_Click(object sender, RoutedEventArgs e) {
			SearchDialog sd = new SearchDialog();
			sd.FindNext += this.searchDialog_FindNext;
			sd.Closed += this.searchDialog_Closed;
			sd.Owner = this;
			sd.Show();
		}

		void searchDialog_FindNext(object sender, EventArgs e) {
			searchText = ((SearchDialog)sender).SearchText;
			if (tr != null)
				tr.ApplyPropertyValue(TextElement.BackgroundProperty, new SolidColorBrush(Colors.White));
			FindAndReplaceManager frm = new FindAndReplaceManager(eddie.Document);
			TextPointer tp = eddie.CaretPosition;
			tr = frm.GetTextRangeFromPosition(ref tp, searchText, FindOptions.None);
			if (tr != null) {
				tr.ApplyPropertyValue(TextElement.BackgroundProperty, new SolidColorBrush(Colors.Yellow));
				eddie.CaretPosition = tp;
			} else
				eddie.CaretPosition = tp.DocumentStart;
		}

		void searchDialog_Closed(object sender, EventArgs ea) {
			if (tr != null)
				tr.ApplyPropertyValue(TextElement.BackgroundProperty, new SolidColorBrush(Colors.White));
		}

		void applicationSettings_SettingChanging(object sender, SettingChangingEventArgs e) {
			if (e.SettingName.Equals("ZoomFactor")) {
				if ((int)e.NewValue > 300) {
					e.Cancel = true;
					MessageBox.Show("Gespeichert wird der Zoom-Faktor " + applicationSettings.ZoomFactor);
				} else
					MessageBox.Show("Gespeichert wird der Zoom-Faktor " + e.NewValue);
			}
		}


		private void Window_Loaded(object sender, RoutedEventArgs e) {
			Keyboard.Focus(eddie);

			try {
				wordWrap = applicationSettings.WordWrap;
				vertSB = applicationSettings.VertSB;
				zoomFactor = applicationSettings.ZoomFactor;
				Title = applicationSettings.AppName;
				activateUserSettings();
			} catch {
				MessageBox.Show("Fehler beim Lesen der Einstellungen\n" +
						"Beim Beenden können keine Einstellungen gespeichert werden.",
						Title, MessageBoxButton.OK, MessageBoxImage.Error);
				applicationSettings = null;
			}
			//Configuration config = ConfigurationManager.OpenExeConfiguration(
			//    ConfigurationUserLevel.PerUserRoamingAndLocal);
			//MessageBox.Show(config.FilePath);
		}

		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			if (applicationSettings != null) {
				applicationSettings.WordWrap = wordWrap;
				applicationSettings.VertSB = vertSB;
				applicationSettings.ZoomFactor = zoomFactor;
				try {
					applicationSettings.Save();
				} catch {
					MessageBox.Show("Fehler beim Sichern der Einstellungen",
						Title, MessageBoxButton.OK, MessageBoxImage.Error);
				}
			}
		}
	}
}
