﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Eddie.Properties;
using System.Configuration;

namespace Eddie {
	/// <summary>
	/// Interaktionslogik für MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window {

		bool wordWrap = true;
		bool vertSB = false;
		int zoomFactor = 100;

		String searchText;
		TextRange tr;

		public MainWindow() {
			InitializeComponent();
			//Title = Settings.Default.AppName;
		}

		void activateUserSettings() {
			if (wordWrap) {
				eddie.HorizontalScrollBarVisibility = ScrollBarVisibility.Disabled;
				eddie.Document.PageWidth = Double.NaN;
			} else {
				eddie.HorizontalScrollBarVisibility = ScrollBarVisibility.Visible;
				eddie.Document.PageWidth = 5000;
			}
			if (vertSB)
				eddie.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;
			else
				eddie.VerticalScrollBarVisibility = ScrollBarVisibility.Disabled;
			eddie.LayoutTransform = new ScaleTransform(zoomFactor / 100, zoomFactor / 100);
		}


		private void miOptions_Click(object sender, RoutedEventArgs e) {
			OptionsDialog od = new OptionsDialog();
			od.Owner = this;
			od.WordWrap = wordWrap;
			od.VertSB = vertSB;
			od.Zoom = zoomFactor;
			if (od.ShowDialog() == true) {
				wordWrap = od.WordWrap;
				vertSB = od.VertSB;
				zoomFactor = od.Zoom;
			}
			activateUserSettings();
		}

		private void miSearch_Click(object sender, RoutedEventArgs e) {
			SearchDialog sd = new SearchDialog();
			sd.FindNext += this.searchDialog_FindNext;
			sd.Closed += this.searchDialog_Closed;
			sd.Owner = this;
			sd.Show();
		}

		private void searchDialog_FindNext(object sender, EventArgs e) {
			searchText = ((SearchDialog)sender).SearchText;
			if (tr != null)
				tr.ApplyPropertyValue(TextElement.BackgroundProperty, new SolidColorBrush(Colors.White));
			FindAndReplaceManager frm = new FindAndReplaceManager(eddie.Document);
			TextPointer tp = eddie.CaretPosition;
			tr = frm.GetTextRangeFromPosition(ref tp, searchText, FindOptions.None);
			if (tr != null) {
				tr.ApplyPropertyValue(TextElement.BackgroundProperty, new SolidColorBrush(Colors.Yellow));
				eddie.CaretPosition = tp;
			} else
				eddie.CaretPosition = tp.DocumentStart;
		}

		private void searchDialog_Closed(object sender, EventArgs ea) {
			if (tr != null)
				tr.ApplyPropertyValue(TextElement.BackgroundProperty, new SolidColorBrush(Colors.White));
		}

		private void Window_Loaded(object sender, RoutedEventArgs e) {
			Keyboard.Focus(eddie);

			// Einstellungen lesen und aktivieren
			wordWrap = Settings.Default.WordWrap;
			vertSB = Settings.Default.VertSB;
			zoomFactor = Settings.Default.ZoomFactor;
			activateUserSettings();

			Left = Settings.Default.WindowPosLeft;
			Top = Settings.Default.WindowPosTop;
			Width = Settings.Default.WindowSizeWidth;
			Height = Settings.Default.WindowSizeHeight;
		}

		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			Settings.Default.WordWrap = wordWrap;
			Settings.Default.VertSB = vertSB;
			Settings.Default.ZoomFactor = zoomFactor;
			Settings.Default.WindowPosLeft = Left;
			Settings.Default.WindowPosTop = Top;
			Settings.Default.WindowSizeWidth = Width;
			Settings.Default.WindowSizeHeight = Height;
			Settings.Default.Save();
		}
	}
}
