﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Net.Sockets;
using System.IO;
using System.Threading;

namespace ChatClientNamespace {
	/// <summary>
	/// Interaktionslogik für MainWindow.xaml
	/// </summary>

	delegate void ProtUpdateDelegate(string s);	
	
	public partial class ChatClient : Window {

		const int PORT = 13000;
		TcpClient tcpClient;
		StreamReader reader;
		StreamWriter writer;
		bool connected;
		ProtUpdateDelegate protokollant;
		System.Reflection.Assembly ass;

		public ChatClient() {
			InitializeComponent();
			ass = System.Reflection.Assembly.GetExecutingAssembly();
			tbMessage.IsEnabled = false;
			cbSend.IsEnabled = false;
			tbUserName.Text = Environment.UserName;
			protokollant = new ProtUpdateDelegate(ProtUpdate);
		}


		void Receive(Object obj) {
			String zeile;
			try {
				while (connected && (zeile = reader.ReadLine()) != null) {
					Dispatcher.BeginInvoke(protokollant, zeile);
					//MessageBox.Show(zeile, "Eingegangen", MessageBoxButton.OK, MessageBoxImage.Information);
					if (zeile == "#Bye!" || zeile == "#EndOfService!")
						return;
				}
			} catch (Exception ex) {
				MessageBox.Show(ex.Message, "Fehler beim Nachrichtenempfang.", MessageBoxButton.OK, MessageBoxImage.Error);
			}
		}

		void ProtUpdate(string s) {
			if (s == "#Bye!")
				cbDisconnect_Click(this, new RoutedEventArgs());
			else if (s == "#EndOfService!") {
				tbProtocol.AppendText(Environment.NewLine + Environment.NewLine + "CHATCLIENT  : " + tbServer.Text +
					" hat die Verbindung beendet" + Environment.NewLine);
				Disconnect();
			} else
				tbProtocol.AppendText(Environment.NewLine + s);
			tbProtocol.ScrollToEnd();
		}

		private void cbConnect_Click(object sender, RoutedEventArgs e) {
			if (connected || tbServer.Text.Length == 0)
				return;
			try {
				tcpClient = new TcpClient(tbServer.Text, PORT);
				reader = new StreamReader(tcpClient.GetStream());
				writer = new StreamWriter(tcpClient.GetStream());
				writer.AutoFlush = true;
				connected = true;
				tbServer.IsEnabled = false;
				cbConnect.IsEnabled = false;
				tbUserName.IsEnabled = false;
				tbMessage.IsEnabled = true;
				cbSend.IsEnabled = true;
				this.Title = "ChatClient (" + ass.GetName().Version + ") verbunden mit " + tbServer.Text;
				// Empfänger-Thread starten
				ThreadPool.QueueUserWorkItem(this.Receive);
				// Beim Serer mit einem geeigneten Namen anmelden
				if (tbUserName.Text.Length > 0)
					writer.WriteLine("#UserName='" + tbUserName.Text.Substring(0, Math.Min(tbUserName.Text.Length, 12)) + "'!");
				else {
					tbUserName.Text = Environment.UserName;
					writer.WriteLine("#UserName='" + Environment.UserName.Substring(0, Math.Min(Environment.UserName.Length, 12)) + "'!");
				}
				//this.AcceptButton = this.cbSend;
				cbSend.IsDefault = true;
				tbMessage.Focus();
			} catch (Exception ex) {
				MessageBox.Show(ex.Message, "Fehler bei der Verbindungsaufnahme", MessageBoxButton.OK, MessageBoxImage.Error);
			}

		}

		private void cbDisconnect_Click(object sender, RoutedEventArgs e) {
			if (!connected)
				return;
			try {
				writer.WriteLine("#Quit!");
				tbProtocol.AppendText(Environment.NewLine + Environment.NewLine + "CHATCLIENT:\tVerbindung zu " + tbServer.Text + " beendet" + Environment.NewLine);
			} catch (Exception ex) {
				MessageBox.Show(ex.Message, "Fehler beim Quit-Kommando", MessageBoxButton.OK, MessageBoxImage.Error);
			} finally {
				Disconnect();
			}
		}

		private void Disconnect() {
			if (!connected)
				return;
			try {
				connected = false;
				// Der Receive-Thread erhält etwas Zeit, sich zu beenden
				Thread.Sleep(500);
				tcpClient.Client.Shutdown(SocketShutdown.Both);
				tcpClient.Close();
				tbServer.IsEnabled = true;
				cbConnect.IsEnabled = true;
				tbUserName.IsEnabled = true;
				tbMessage.Text = "";
				tbMessage.IsEnabled = false;
				cbSend.IsEnabled = false;
				this.Title = "ChatClient (" + ass.GetName().Version + ")";
				cbConnect.IsDefault = true;
				tbServer.Focus();
			} catch (Exception ex) {
				MessageBox.Show(ex.Message, "Fehler beim Schließen der Verbindung", MessageBoxButton.OK, MessageBoxImage.Error);
			}
		}

		private void cbSend_Click(object sender, RoutedEventArgs e) {
			if (tbMessage.Text.Length > 0)
				try {
					writer.WriteLine(tbMessage.Text);
					// gesendeten Text markieren
					tbMessage.SelectionStart = 0;
					tbMessage.SelectionLength = tbMessage.Text.Length;
					tbMessage.Focus();
				} catch (Exception ex) {
					MessageBox.Show(ex.Message, "Fehler beim Senden der Nachricht", MessageBoxButton.OK, MessageBoxImage.Error);
				}
		}

		private void cbEnd_Click(object sender, RoutedEventArgs e) {
			if (connected)
				cbDisconnect_Click(new object(), new RoutedEventArgs());
			Environment.Exit(0);
		}

		private void Window_Activated(object sender, EventArgs e) {
			// Beim Erscheinen des Formulars soll die Textbox mit dem Servernamen den Fokus haben.
			tbServer.Focus();
			cbConnect.IsDefault = true;
		}

		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			if (connected)
				cbDisconnect_Click(new object(), new RoutedEventArgs());
		}

	}
}
